//------------------------------ $Keywords ----------------------------------
// SelfImage - Windows disk image backup
// SelfImage_Utility.cpp - Utility and miscelaneous functions
// Copyright 2005, Kurt Fitzner <kfitzner@excelcia.org>
//---------------------------------------------------------------------------
// This file is part of SelfImage.
//
// SelfImageis free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License (Version 2) as
// published by the Free Software Foundation.
//
// SelfImage is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// VCS: $Version: 0 $ $Revision: 6 $
/*
$History: **** V 0.1 by kfitzner ****
$History: * selfimage_utility.cpp - 2005-07-31 8:29:45 AM - 1963 Bytes
$History: * selfimage_utility.h - 2005-07-31 8:21:56 AM - 1395 Bytes
$History: * Initial check-in
$History: **** V 0.2 by kfitzner ****
$History: * selfimage_utility.cpp - 2005-08-01 1:56:17 PM - 2833 Bytes
$History: * selfimage_utility.h - 2005-07-31 9:14:14 AM - 1439 Bytes
$History: * Add GetLastErrorMessage().
$History: **** V 0.3 by kfitzner ****
$History: * selfimage_utility.cpp - 2005-08-01 3:11:51 PM - 3827 Bytes
$History: * selfimage_utility.h - 2005-08-01 2:13:58 PM - 1506 Bytes
$History: * Add MakeByteSizeLabel().
$History: **** V 0.4 by kfitzner ****
$History: * selfimage_utility.cpp - 2005-08-06 6:29:09 PM - 13684 Bytes
$History: * selfimage_utility.h - 2005-08-06 6:13:00 PM - 6700 Bytes
$History: * Add support for unmounted volumes
$History: **** V 0.5 by kfitzner ****
$History: * selfimage_utility.cpp - 2005-08-17 7:51:35 AM - 13878 Bytes
$History: * selfimage_utility.h - 2005-08-17 7:49:02 AM - 6635 Bytes
$History: * Change licensing - only version 2 of the GPL, no later versions
$History: **** Latest ** V 0.6 by kfitzner ** 2005-11-07 2:41:16 AM ****
$History: * Changes for 0.2 - too many to note
*/
//----------------------------  $NoKeywords ---------------------------------

//---------------------------------------------------------------------------
// File Notes:
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "SelfImage_Utility.h"
//---------------------------------------------------------------------------
static HMODULE ntdll = NULL;
static NtOpenDirectoryObject_t     NtOpenDirectoryObject     = NULL;
static NtQueryDirectoryObject_t    NtQueryDirectoryObject    = NULL;
static NtOpenSymbolicLinkObject_t  NtOpenSymbolicLinkObject  = NULL;
static NtQuerySymbolicLinkObject_t NtQuerySymbolicLinkObject = NULL;
static RtlInitUnicodeString_t      RtlInitUnicodeString      = NULL;
static NtCreateFile_t              NtCreateFile              = NULL;
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall InitializeUtilityFunctions(void) {
  ntdll = GetModuleHandle("ntdll.dll");
  if (ntdll) {
    NtOpenDirectoryObject     = (NtOpenDirectoryObject_t)    GetProcAddress(ntdll, "NtOpenDirectoryObject");
    NtQueryDirectoryObject    = (NtQueryDirectoryObject_t)   GetProcAddress(ntdll, "NtQueryDirectoryObject");
    NtOpenSymbolicLinkObject  = (NtOpenSymbolicLinkObject_t) GetProcAddress(ntdll, "NtOpenSymbolicLinkObject");
    NtQuerySymbolicLinkObject = (NtQuerySymbolicLinkObject_t)GetProcAddress(ntdll, "NtQuerySymbolicLinkObject");
    NtCreateFile              = (NtCreateFile_t)             GetProcAddress(ntdll, "NtCreateFile");
    RtlInitUnicodeString      = (RtlInitUnicodeString_t)     GetProcAddress(ntdll, "RtlInitUnicodeString");
    HaveNTCalls = true;
  }  // if (ntdll)
}  // void __fastcall InitializeUtilityFunctions(void)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Get the last Windows error and format the message into an AnsiString
//
AnsiString __fastcall GetLastErrorMessage(void)
{
  char *buffer;
  AnsiString retval;

  FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
    (LPTSTR) &buffer, 0, NULL );
  retval = String(buffer).Trim();
  LocalFree(buffer);

  return(retval);
}  // inline AnsiString GetLastErrorMessage(void)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Convert a number into a normalized string with a KB/MB/GB/TB suffix
// appropriate for a byte capacity label
//
AnsiString __fastcall MakeByteLabel(long double ddByteCount)
{
  AnsiString retval;

  if (ddByteCount > 1099511627776.0)
    retval.sprintf("%3.3LfTB", ddByteCount / 1099511627776.0);
  else if (ddByteCount > 1073741824.0)
    retval.sprintf("%3.3LfGB", ddByteCount / 1073741824.0);
  else if (ddByteCount > 1048576.0)
    retval.sprintf("%3.3LfMB", ddByteCount / 1048576.0);
  else if (ddByteCount > 1024.0)
    retval.sprintf("%3.3LfKB", ddByteCount / 1024.0);
  else retval.sprintf("%Lf",ddByteCount);

  return(retval);
}  // AnsiString __fastcall MakeByteSizeLabel(__int64 &nnByteCount)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Get the contents of an NT directory object - this uses some undocumented
// calls from ntdll.dll
//
TStringList * __fastcall GetNTDirectoryObjectContents(WideString Directory)
{
  TStringList *Entries;
  UNICODE_STRING usDir;
  OBJECT_ATTRIBUTES oa;
  HANDLE hDeviceDir;
  NTSTATUS nStatus;
  OBJDIR_INFORMATION *DirInfo;
  DWORD index;
  AnsiString Error;

  if (!HaveNTCalls)
    return NULL;

  RtlInitUnicodeString(&usDir, Directory);
  oa.Length = sizeof(OBJECT_ATTRIBUTES);
  oa.ObjectName = &usDir;
  oa.Attributes = OBJ_CASE_INSENSITIVE;
  oa.SecurityDescriptor = NULL;
  oa.SecurityQualityOfService = NULL;
  oa.RootDirectory = 0;
  nStatus = NtOpenDirectoryObject(&hDeviceDir, STANDARD_RIGHTS_READ | DIRECTORY_QUERY, &oa);
  if (!NT_SUCCESS(nStatus))
    return NULL;
  DirInfo = (OBJDIR_INFORMATION *)malloc(2048);
  index = 0;
  Entries = new TStringList;
  while (NT_SUCCESS(NtQueryDirectoryObject(hDeviceDir, DirInfo, 1024, true, false, &index, NULL)))
    Entries->Add(Directory + "\\" + String(DirInfo->ObjectName.Buffer));
  CloseHandle(hDeviceDir);
  free(DirInfo);
  if (Entries->Count)
    return Entries;
  else {  // if (!Entries->Count)
    delete Entries;
    return NULL;
  }  // if (!Entries->Count)
}   // TStringList * __fastcall GetNTDirectoryObjectContents(WideString Directory)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
AnsiString __fastcall GetNTLinkDestination(WideString Source)
{
  UNICODE_STRING usName;
  OBJECT_ATTRIBUTES oa;
  NTSTATUS nStatus;
  HANDLE hLink;
  wchar_t *buffer;
  AnsiString retval;

  RtlInitUnicodeString(&usName, Source);
  oa.Length = sizeof(OBJECT_ATTRIBUTES);
  oa.RootDirectory = 0;
  oa.ObjectName = &usName;
  oa.Attributes = OBJ_CASE_INSENSITIVE;
  oa.SecurityDescriptor = NULL;
  oa.SecurityQualityOfService = NULL;
  nStatus = NtOpenSymbolicLinkObject(&hLink, SYMBOLIC_LINK_QUERY, &oa);
  if (NT_SUCCESS(nStatus)) {
    buffer = (wchar_t *)malloc(2048);
    usName.Length = 0;
    usName.MaximumLength = 1024;
    usName.Buffer = buffer;
    nStatus = NtQuerySymbolicLinkObject(hLink, &usName, NULL);
    usName.Buffer[usName.Length/2] = 0;
    if (NT_SUCCESS(nStatus))
      retval = String(usName.Buffer);
    free(buffer);
  }  // if (NTSUCCESS(nStatus))
  return retval;
}  // AnsiString __fastcall GetNTLinkDestination(AnsiString Source)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// This is a wrapper function around the native NtCreateFile() function.
// Officially, NtCreateFile() is undocumented, but it is widely used.
//
HANDLE NTOpen(WideString FileName, ACCESS_MASK DesiredAccess, ULONG FileAttributes, ULONG ShareAccess, ULONG CreateDisposition, ULONG CreateOptions)
{
  UNICODE_STRING usFileName;
  OBJECT_ATTRIBUTES oa;
  HANDLE hFile;
  IO_STATUS_BLOCK ios;
  NTSTATUS nStatus;

  if (!HaveNTCalls)
    return INVALID_HANDLE_VALUE;
  RtlInitUnicodeString(&usFileName, FileName);
  oa.Length = sizeof(oa);
  oa.RootDirectory = 0;
  oa.ObjectName = &usFileName;
  oa.Attributes = OBJ_CASE_INSENSITIVE;
  oa.SecurityDescriptor = NULL;
  oa.SecurityQualityOfService = NULL;
  nStatus = NtCreateFile(&hFile, DesiredAccess, &oa, &ios, NULL, FileAttributes, ShareAccess, CreateDisposition, CreateOptions, NULL, 0);
  if (NT_SUCCESS(nStatus))
    return hFile;
  else
    return INVALID_HANDLE_VALUE;
}  // HANDLE XpCreateFile()
//---------------------------------------------------------------------------




